local AddOn, _DUF = ...
local strfind = string.find
local Classbar_BaseName = "DUF_ClassFrame"
local ceil = math.ceil

local RUNE_KEY_BY_SPEC = {
	[1] = "Blood",
	[2] = "Frost",
	[3] = "Unholy",
};

-----------------------------------------------------------------------------------
-- Start Semlar Cooldown code
-----------------------------------------------------------------------------------

-- Some math stuff
local cos, sin, pi2, halfpi = math.cos, math.sin, math.rad(360), math.rad(90)
local function Transform(tx, x, y, angle, aspect) -- Translates texture to x, y and rotates about its center
	local c, s = cos(angle), sin(angle)
	local y, oy = y / aspect, 0.5 / aspect
	local ULx, ULy = 0.5 + (x - 0.5) * c - (y - oy) * s, (oy + (y - oy) * c + (x - 0.5) * s) * aspect
	local LLx, LLy = 0.5 + (x - 0.5) * c - (y + oy) * s, (oy + (y + oy) * c + (x - 0.5) * s) * aspect
	local URx, URy = 0.5 + (x + 0.5) * c - (y - oy) * s, (oy + (y - oy) * c + (x + 0.5) * s) * aspect
	local LRx, LRy = 0.5 + (x + 0.5) * c - (y + oy) * s, (oy + (y + oy) * c + (x + 0.5) * s) * aspect
	tx:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy)
end

local function SetValue(self, value)
	-- Correct invalid ranges, preferably just don't feed it invalid numbers
	if value > 1 then 
		value = 1
	elseif value < 0 then 
		value = 0 
	end
	-- Reverse our normal behavior
	if self._reverse then
		value = 1 - value
	end
	-- Determine which quadrant we're in
	local q, quadrant = self._clockwise and (1 - value) or value -- 4 - floor(value / 0.25)
	if q >= 0.75 then
		quadrant = 1
	elseif q >= 0.5 then
		quadrant = 2
	elseif q >= 0.25 then
		quadrant = 3
	else
		quadrant = 4
	end
	if self._quadrant ~= quadrant then
		self._quadrant = quadrant
		-- Show/hide necessary textures if we need to
		if self._clockwise then
			for i = 1, 4 do
				self._textures[i]:SetShown(i < quadrant)
			end
		else
			for i = 1, 4 do
				self._textures[i]:SetShown(i > quadrant)
			end
		end
		-- Move scrollframe/wedge to the proper quadrant
		self._scrollframe:Hide();
		self._scrollframe:SetAllPoints(self._textures[quadrant])
		self._scrollframe:Show();
	end
	-- Rotate the things
	local rads = value * pi2
	if not self._clockwise then 
		rads = -rads + halfpi 
	end
	Transform(self._wedge, -0.5, -0.5, rads, self._aspect)
--	self._rotation:SetDuration(0.000001)
	self._rotation:SetDuration(0)
	self._rotation:SetEndDelay(2147483647)
	self._rotation:SetOrigin('BOTTOMRIGHT', 0, 0)
	self._rotation:SetRadians(-rads);
	self._group:Play();
end

local function SetClockwise(self, clockwise)
	self._clockwise = clockwise
end

local function SetReverse(self, reverse)
	self._reverse = reverse
end

local function OnSizeChanged(self, width, height)
	self._wedge:SetSize(width, height) -- it's important to keep this texture sized correctly
	self._aspect = width / height -- required to calculate the texture coordinates
end

-- Creates a function that calls a method on all textures at once
local function CreateTextureFunction(func, self, ...)
	return function(self, ...)
		for i = 1, 4 do
			local tx = self._textures[i]
			tx[func](tx, ...)
		end
		self._wedge[func](self._wedge, ...)
	end
end

-- Pass calls to these functions on our frame to its textures
local TextureFunctions = {
	SetTexture = CreateTextureFunction('SetTexture'),
	SetBlendMode = CreateTextureFunction('SetBlendMode'),
	SetVertexColor = CreateTextureFunction('SetVertexColor'),
	SetAlpha = CreateTextureFunction('SetAlpha'),
}

local function CreateCooldown(parent, name, settings)
	local cooldown = CreateFrame('Frame', name, parent)
	cooldown:Hide()
	cooldown:SetAllPoints()
	cooldown.timerFrame = CreateFrame("Frame", nil, cooldown)
	cooldown.timerFrame:SetFrameLevel(cooldown:GetFrameLevel()+2) 
	cooldown.timerFrame:SetAllPoints()
	cooldown.timerFrame.timer = cooldown.timerFrame:CreateFontString()
--	cooldown.timerFrame.timer:SetFontObject(GameFontHighlightSmall)
	cooldown.timerFrame.timer:SetFont(settings.cooldownfont, settings.cooldownfontsize, "OUTLINE")
	if settings.cooldownfonthide then
		cooldown:Hide()
	end
	cooldown.timerFrame.timer:SetPoint("CENTER", cooldown)
	cooldown.timerFrame.timer:SetAllPoints()
	cooldown.timerFrame.timer:SetJustifyH("CENTER")
	cooldown.timerFrame.timer:SetJustifyV("CENTER")
--	cooldown.timer:SetDrawLayer("BORDER")
	-- ScrollFrame clips the actively animating portion of the cooldown
	local scrollframe = CreateFrame('ScrollFrame', nil, cooldown)
	scrollframe:SetPoint('BOTTOMLEFT', cooldown, 'CENTER')
	scrollframe:SetPoint('TOPRIGHT')
	cooldown._scrollframe = scrollframe

	local scrollchild = CreateFrame('frame', nil, scrollframe)
	scrollframe:SetScrollChild(scrollchild)
	scrollchild:SetAllPoints(scrollframe)
	
	-- Wedge thing
	local wedge = scrollchild:CreateTexture()
	wedge:SetPoint('BOTTOMRIGHT', cooldown, 'CENTER')
	cooldown._wedge = wedge

	-- Top Right
	local trTexture = cooldown:CreateTexture()
	trTexture:SetPoint('BOTTOMLEFT', cooldown, 'CENTER')
	trTexture:SetPoint('TOPRIGHT')
	trTexture:SetTexCoord(0.5, 1, 0, 0.5)

	-- Bottom Right
	local brTexture = cooldown:CreateTexture()
	brTexture:SetPoint('TOPLEFT', cooldown, 'CENTER')
	brTexture:SetPoint('BOTTOMRIGHT')
	brTexture:SetTexCoord(0.5, 1, 0.5, 1)

	-- Bottom Left
	local blTexture = cooldown:CreateTexture()
	blTexture:SetPoint('TOPRIGHT', cooldown, 'CENTER')
	blTexture:SetPoint('BOTTOMLEFT')
	blTexture:SetTexCoord(0, 0.5, 0.5, 1)

	-- Top Left
	local tlTexture = cooldown:CreateTexture()
	tlTexture:SetPoint('BOTTOMRIGHT', cooldown, 'CENTER')
	tlTexture:SetPoint('TOPLEFT')
	tlTexture:SetTexCoord(0, 0.5, 0, 0.5)

	-- /4|1\ -- Clockwise texture arrangement
	-- \3|2/ --

	cooldown._textures = {trTexture, brTexture, blTexture, tlTexture}
	cooldown._quadrant = nil -- Current active quadrant
	cooldown._clockwise = true -- fill clockwise
	cooldown._reverse = false -- Treat the provided value as its inverse, eg. 75% will display as 25%
	cooldown._aspect = 1 -- aspect ratio, width / height of cooldown frame
	cooldown:HookScript('OnSizeChanged', OnSizeChanged)

	for method, func in pairs(TextureFunctions) do
		cooldown[method] = func
	end

	cooldown.SetClockwise = SetClockwise
	cooldown.SetReverse = SetReverse
	cooldown.SetValue = SetValue

	local group = wedge:CreateAnimationGroup()
	group:SetScript('OnFinished', function() group:Play() end);
	local rotation = group:CreateAnimation('Rotation')
	cooldown._rotation = rotation
	cooldown._group = group;

	cooldown:SetClockwise(false)
	cooldown:SetReverse(true)

	cooldown.timespent = 0
	cooldown.totaltime = 0
	cooldown:SetScript('OnUpdate', function(self, elapsed)
			if self.start > GetTime() then return end
			self.timespent = self.timespent + elapsed
			if self.timespent >= self.totaltime then
			    self.timespent = 0
			    self.totaltime = 0
			    self:Hide()
			end
			local value
			if self.totaltime == 0 then
				value = 0
			else 
				value = self.timespent / self.totaltime
			end
			local cdTime = ceil(self.totaltime-self.timespent)
			if cdTime == 0 then
				cdTime = ""
			end
			cooldown.timerFrame.timer:SetText(cdTime)
			self:SetValue(value)
		end)
	function cooldown:StartPlay(timetoplay, start)
		self.totaltime = timetoplay
		self.timespent = 0
		self.start = start
		self:Show()
	end

	return cooldown
end
-----------------------------------------------------------------------------------
-- End Semlar Cooldown code
-----------------------------------------------------------------------------------

local function MonkStaggerUpdate(self, elapsed)
	if (not self.timer) then 
		self.timer = self.updatespeed
		self.speed = self.timer
	end
	self.timer = self.timer - elapsed;
	if self.timer > 0 then return end
	self.timer = self.speed
	local currstagger = UnitStagger("player")
	local maxhealth = UnitHealthMax("player")
	if not self.laststagger or currstagger > self.laststagger then 
		self.laststagger = currstagger
	end
	if not currstagger or not maxhealth or maxhealth == 0 then return end
	local value = currstagger / maxhealth * 100
	local value2 = 0
	if DUF_Settings[DUF_INDEX].player.MONK1.fade and self.laststagger < maxhealth then
		value2 = self.laststagger / maxhealth * 100
	end
	if self.laststagger > currstagger and value2 > 0 then
		self.laststaggeralpha = 1
	end
	local width = DUF_Settings[DUF_INDEX].player.MONK1.width
	if DUF_Settings[DUF_INDEX].player.MONK1.resizemax and self.lasthealthmax ~= maxhealth then
		self.lasthealthmax = maxhealth
		local percent = maxhealth / currstagger
		width = width * percent
		local padding = DUF_Settings[DUF_INDEX].player.MONK1.bgpadding * 2
		self.dynamicsize = percent
		self.Background:SetWidth(width + padding)
	end
	self.laststagger = currstagger
	if self:GetParent().config then
		value = 100
		value2 = 100
	end
	if value <= 0 then
		if not self.hidden then
			self:GetParent():SetAlpha(self:GetParent().oocalpha)
			self.hidden = true
		end
	else
		if self.hidden then
			self:GetParent():SetAlpha(self:GetParent().showalpha)
			self.hidden = nil
		end
	end
	if self.lastvalue ~= value then
		self.lastvalue = value
	else
		return
	end
	if DUF_Settings[DUF_INDEX].player.MONK1.fill then
		_DUF:StatusBar_SetValue(self.Bar1, 100 - value, 1, width)
		_DUF:StatusBar_SetValue(self.Bar2, 0, 1, width)
	else
		_DUF:StatusBar_SetValue(self.Bar1, value, 1, width)
		_DUF:StatusBar_SetValue(self.Bar2, value2, 1, width)
	end
end

local function DUF_ClassFrame_Update(self, runeIndex, isEnergize)
	if self.type ~= "rune" then return end
	if runeIndex then
		if runeIndex and runeIndex >= 1 and runeIndex <= self.maxShown  then 
			local rune = self.Runes[runeIndex]
--			local cooldown = runeButton.Cooldown
			local start, duration, runeReady = GetRuneCooldown(runeIndex)
			if not runeReady  then
				if start then
					self.Deactivate(rune, start, duration)
--					CooldownFrame_Set(cooldown, start, duration, true, true);
				end
--				runeButton.energize:Stop();
			else
				self.Activate(rune)

--				cooldown:Hide();
--				if (not isEnergize and not runeButton.energize:IsPlaying()) then 
--					runeButton.shine:SetVertexColor(1, 1, 1);
--					RuneButton_ShineFadeIn(runeButton.shine)
--				end
			end
--			if isEnergize  then
--				runeButton.energize:Play();
--			end
		else 
			assert(false, "Bad rune index")
		end

	else
		local toShow = UnitPower(self:GetParent().unit, self.power)
		local maxShown = self.maxShown
		if self.config then
			toShow = self.total
			self.lastShown = 0
			self.wasconfiged = true
		end
		if toShow and (not self.lastShown or self.lastShown > toShow) then
			self.lastShown = 0
		end

		if toShow > 0 then
			if not self:IsShown() then
				self:Show()
			end
			for i=1, maxShown do
				local rune = self.Runes[i] -- self.ComboPoints[comboIndex]
				if i <= toShow then
					if i >= self.lastShown then
						-- Fade in the highlight and set a function that triggers when it is done fading
						self.Activate(rune)
					end
				else
					self.Deactivate(rune)
				end
			end
		else
			local max = self.maxShown
			if self.wasconfiged or self.anim == "highlight" then
				max = self.total
				self.wasconfiged = nil
				self.lastShown = 0
				_DUF:SetupDruid(self)
			end
			for i=1, max do 
				self.Deactivate(self.Runes[i])
			end
		end
		self.lastShown = toShow
	end
end

local function DUF_ClassFrame_UpdateMax(self)
	if self.type ~= "rune" then return end
	local maxShown = self.maxShown
	self.maxShown = UnitPowerMax("player", self.power)

if not self.config and self.maxShown > self.total then
   DL_Debug(UnitClass("player") .. "s now have " .. self.maxShown .. " class bar points instead of " .. self.total .. "!")
end
	if self.config then
		self.maxShown = self.total
	end
	if maxShown ~= self.maxShown then
		for i=1, #self.Runes do
			if i > self.maxShown then -- and not self.config then
				self.Runes[i]:Hide()
			else
				self.Runes[i]:Show()
			end
		end
	end
	local shown = self.total
	if self.centeralign then
		shown = self.maxShown
	end
	local width, height = self.Runes[shown]:GetWidth(), self.Runes[1]:GetHeight()
	if self.direction == 1 then
		width = self.Runes[shown]:GetRight() - self.Runes[1]:GetLeft()
	elseif self.direction == 2 then
		width = self.Runes[1]:GetRight() - self.Runes[shown]:GetLeft()
	elseif self.direction == 3 then
		height = self.Runes[shown]:GetTop() - self.Runes[1]:GetBottom()
	else --4 
		height = self.Runes[1]:GetTop() - self.Runes[shown]:GetBottom()
	end
	self:SetSize(width, height)
end

local function DUF_ClassFrame_OnEvent(self, event, ...)
	local arg1 = ...
	if event == "PLAYER_TARGET_CHANGED" then
		local alpha = self.oocalpha 
		if UnitExists("target") then
			alpha = self.showalpha
		end
		self:SetAlpha(alpha)
	elseif event == "UNIT_POWER_FREQUENT" then
		local unit = ...
		if unit == DUF_PlayerFrame.unit then
			DUF_ClassFrame_Update(self)
		end
	elseif event == "RUNE_POWER_UPDATE" then
		local runeIndex, isEnergize = ...
		DUF_ClassFrame_Update(self, runeIndex, isEnergize)
	elseif event == "UNIT_DISPLAYPOWER" then
		_DUF:SetupDruid(self)
--	elseif event == "RUNE_TYPE_UPDATE" then maybe later, it's just a flash ATM
	elseif event == "PLAYER_SPECIALIZATION_CHANGED" then
		arg1 = ...
		if arg1 ~= "player" then return end
		DUF_Initialize_ClassBar(self.class)
	elseif event == "UNIT_MAXPOWER" or event == "PLAYER_ENTERING_WORLD" then
		DUF_Initialize_ClassBar(self.class)
		if _DUF:SetupDruid(self) then
			DUF_ClassFrame_Update(self)
		end
	elseif event ==  "PLAYER_LEVEL_UP" then
		if arg1 >= self.showlevel then
			DUF_ClassFrame_UpdateMax(self)
			DUF_ClassFrame_Update(self)
			self:UnregisterEvent("PLAYER_LEVEL_UP");
		end
	end
end

local function RuneFadeEnd(self)
--	self:SetScript("OnUpdate", nil)
	self.Texture:SetAlpha(1)
	if self.Cooldown then
		self.Cooldown:Hide()
	end
end

local function RuneFadeStart(self, start, duration)
	if not start then return end
--	self.Texture:SetAlpha(self:GetParent().hidealpha)
--	self:SetScript("OnUpdate", nil)
	if self.Cooldown then 
		if not self.Cooldown:IsShown() then
			self.Texture:SetAlpha(self.calpha)
			self.Cooldown:SetAlpha(self.cdalpha)
			self.Cooldown:StartPlay(duration, start)
		end
	else
		self.Texture:SetAlpha(self:GetParent().oocalpha)
	end
end

local function RuneHide(self)
--	if self.anim == "show" or self.anim == "highlight" then
		if self.hasBorder then
			self.Texture:SetAlpha(self:GetParent().hidealpha)
--			self.Texture:SetAlpha(self:GetParent().oocalpha)
		else
			self:SetAlpha(self:GetParent().hidealpha)
--			self:SetAlpha(self:GetParent().oocalpha)
		end
--	end
end

local function RuneShow(self)
--	if self.anim == "show" or self.anim == "highlight" then
		if self.hasBorder then
			self.Texture:SetAlpha(1)
		else
			self:SetAlpha(1)
		end
--	end
end

local function GetTexturePath(texture, default)
	if not default then
		return _DUF:GetResourcePath(texture)
	end
	return texture
end

local function CreateRune(name, parent, settings)
	local rune = CreateFrame("Frame", name, parent)
	rune.Border = rune:CreateTexture("$parent_Border", "ARTWORK")
	rune.Texture = rune:CreateTexture("$parent_Texture", "ARTWORK")
	rune.Texture:SetAllPoints()
	rune:EnableMouse(false)
	rune:EnableMouseWheel(false)
	rune.moveparent = true
	rune:RegisterForDrag("LeftButton", "RightButton")
	rune:SetScript("OnDragStart", DUF_Element_OnDragStart)
	rune:SetScript("OnDragStop", DUF_Element_OnDragStop)
	if parent.anim == "cooldown" then
		rune.Cooldown = CreateCooldown(rune, name.."Cooldown", settings)
		if settings.cooldownspeccolors then
			rune.Cooldown:SetTexture("Interface/PlayerFrame/DK-"..RUNE_KEY_BY_SPEC[GetSpecialization()].."-Rune-CDFill")
		else
			rune.Cooldown:SetTexture("interface/addons/DiscordUnitFrames/Icons/Cooldown")
		end
		rune.cdalpha = 1
		rune.calpha = 0.65
	end
	return rune
end

local function SetTexture(frame, parent, settings, checkpath)
	local texture, topxos, topyos, bottomxos, bottomyos
	local left, right, top, bottom =  0, 1, 0, 1
	local atlas = settings.atlas
	local useatlassize = settings.useatlassize
	texture = settings.texture
	if checkpath then
		texture = _DUF:GetResourcePath(texture)
	end
	if atlas then
		frame:SetAtlas(texture, useatlassize)
	else
		frame:SetTexture(texture)
	end
	if settings.texcoords then
		left, right, top, bottom = settings.texcoords.left, settings.texcoords.right, settings.texcoords.top, settings.texcoords.bottom
	end
	frame:SetTexCoord(left, right, top, bottom)
	if settings.offsets then
		topxos, topyos, bottomxos, bottomyos = settings.offsets.topxos, settings.offsets.topyos, settings.offsets.bottomxos, settings.offsets.bottomyos
	end
	frame:ClearAllPoints()
	if not topxos then
		frame:SetAllPoints()
	else
		frame:SetPoint("TOPLEFT", parent, "TOPLEFT", topxos, topyos)
		frame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", bottomxos, bottomyos)
	end
	if settings.level then
		frame:SetDrawLayer(settings.level)
	end
	if not settings.defaultcolor and settings.color then
		frame:SetVertexColor(settings.color.r, settings.color.g, settings.color.b)
	end
	if settings.blend then
		frame:SetBlendMode(settings.blend)
	else
		frame:SetBlendMode("BLEND")
	end
	if settings.width then
		frame:SetWidth(settings.width)
	end
	if settings.height then
		frame:SetHeight(settings.height)
	end
end

local function GetSettings(settings, runeindex)
	if #settings == 0
		then return nil -- no borders
	end
	local settingindex = runeindex
	if #settings == 1 then
		settingindex = 1
	end
	if settingindex > 1 and (not settings[settingindex].texture or settings[settingindex].texture == "") then
		settingindex = 1
	end
	return settings[settingindex]
end

local function UpdateTextures(frame, settings, class, runeindex, max)
	local texturesettings, bordersettings, checkpath, hasBorder
	local default = true
	frame:SetSize(settings.width, settings.height)
	frame.Texture:SetVertexColor(settings.color.r, settings.color.g, settings.color.b)
	frame.Border:SetVertexColor(settings.color.r, settings.color.g, settings.color.b)
	local count = #settings.textures
	if count > 0 and (settings.textures[1].texture and settings.textures[1].texture ~= "") then
		default = nil
	end
	if not default then
		checkpath = true
		texturesettings = GetSettings(settings.textures, runeindex)
		bordersettings = GetSettings(settings.borders, runeindex)
	else
		texturesettings = GetSettings(_DUF.CLASS_BAR_DEFAULTS[class].textures, runeindex)
		bordersettings = GetSettings(_DUF.CLASS_BAR_DEFAULTS[class].borders, runeindex)
	end
	SetTexture(frame.Texture, frame, texturesettings, checkpath)
	if bordersettings then
		SetTexture(frame.Border, frame, bordersettings, checkpath)
		frame.hasBorder = true
	else
		frame.Border:SetTexture("")
	end

	if frame.Cooldown and frame:GetParent().anim == "cooldown" then
		if settings.cooldownspeccolors then
			frame.Cooldown:SetTexture("Interface/PlayerFrame/DK-"..RUNE_KEY_BY_SPEC[GetSpecialization()].."-Rune-CDFill")
		else
			frame.Cooldown:SetTexture("interface/addons/DiscordUnitFrames/Icons/Cooldown")
		end
		frame.cdalpha = 1
		frame.calpha = 0.65
		if settings.numtextures and settings.numtextures > 0 and settings.cooldownalpha then
			frame.cdalpha = settings.cooldownalpha
			frame.calpha = settings.cooledalpha
			if settings.cooldowntexture and settings.cooldowntexture ~= "" then
				frame.Cooldown:SetTexture(GetTexturePath(settings.cooldowntexture))
			end
			frame.Cooldown:SetVertexColor(settings.cooldowncolor.r, settings.cooldowncolor.g, settings.cooldowncolor.b)
		end
		frame.Cooldown.timerFrame.timer:SetFont(settings.cooldownfont, settings.cooldownfontsize, "OUTLINE")
		frame.Cooldown.timerFrame.timer:ClearAllPoints()
--		frame.Cooldown.timerFrame.timer:SetPoint("CENTER", frame.Cooldown.timerFrame, 0, settings.cooldownfontsize/5)
		frame.Cooldown.timerFrame.timer:SetPoint("CENTER", frame.Cooldown.timerFrame)
		if settings.cooldownfonthide then
			frame.Cooldown.timerFrame:Hide()
		else
			frame.Cooldown.timerFrame:Show()
		end
	end

end

local function CooldownStart(self)
	_DUF:SetCooldown(self)	
end

local function CooldownEnd(self)
end

function _DUF:SetupDruid(self)
	local _, class = UnitClass("player")
	if class ~= "DRUID" then
--		self:UnregisterEvent("UNIT_DISPLAYPOWER")
		return
	end
--	self:RegisterUnitEvent("UNIT_DISPLAYPOWER", "player")
	local showBar
	local powerType = UnitPowerType("player");
	if powerType == self.displaypower then
		showBar = true
		self:RegisterUnitEvent("UNIT_POWER_FREQUENT", "player")
		self:RegisterUnitEvent("UNIT_MAXPOWER", "player")
	else
		self:UnregisterEvent("UNIT_POWER_FREQUENT")
		self:UnregisterEvent("UNIT_MAXPOWER")
	end
	if showBar then
		self:Show()
		DUF_ClassFrame_UpdateMax(self)
		DUF_ClassFrame_Update(self)
	else
		self:Hide()
	end
	return true
end

----------------------- GLOBAL ---------------------------------------------------------------------------------------------
function DUF_Initialize_ClassBar(class, configspec)
	local bar = _G[Classbar_BaseName]
--	if bar then 
--		bar:Hide()
--	end
	local defaultspec = DUF_CLASS_BAR_SPECS[class]
	if not defaultspec then 
		if bar then 
			bar:UnregisterAllEvents()
			bar:SetScript("OnUpdate", nil)
			bar:SetScript("OnEvent", nil)
			bar:Hide()
		end
		return 
	end
	local spec = configspec or GetSpecialization()
	local settings
	if type(defaultspec[spec]) == "table" then
		defaultspec = defaultspec[spec]
		settings = DUF_Settings[DUF_INDEX].player[class..spec]
	else
		settings = DUF_Settings[DUF_INDEX].player[class]
	end
	if not settings then return end
	if not bar then
		bar = CreateFrame("Frame", Classbar_BaseName, DUF_PlayerFrame, "DUF_UnsecureElementTemplate")
		bar.subindex = nil -- Required because DUF_Element_OnLoad will add a subindex for any frame with "Bar" in its name
		bar.texture = bar:CreateTexture() -- for some reason adding a texture effects centering
		bar:EnableMouse(false)
		bar:EnableMouseWheel(false)
--		bar.texture:SetAllPoints()
	end
	bar.maxShown = nil
	bar.total = nil
	bar.lastShown = nil
	bar.class = class
	bar.unit = "player"
	bar.type = defaultspec.type
	bar.power = defaultspec.power
	bar.displaypower = defaultspec.displaypower
	bar.hide = settings.hide
	bar.oocalpha = settings.oocalpha or 0
	bar.hidealpha = defaultspec.hidealpha
	bar.showalpha = settings.alpha
	bar.config = DUF_OPTIONS_VISIBLE
	local showBar = (bar.config or DUF_CLASS_BAR_SPECS[class][spec]) and not bar.hide
	bar:UnregisterAllEvents()
	bar:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	if not showBar then
		bar:Hide()
		return
	end
	_DUF:DUF_Initialize_Element("player", bar, settings)
	if DUF_CLASS_CENTERS[settings.attachto] and settings.centeralign then
		bar.centeralign = true
	else
		bar.centeralign = nil
	end
	bar.direction = settings.direction
	bar.anim = _DUF.CLASS_BAR_DEFAULTS[class].animation
	if bar.anim == "show" or bar.anim == "highlight"then
		bar.Activate = RuneShow
		bar.Deactivate = RuneHide
	elseif bar.anim == "cooldown" then		
		bar.Activate = RuneFadeEnd
		bar.Deactivate = RuneFadeStart
	end
	local point, toPoint, spacingX, spacingY = _DUF:DUF_Get_Direction(settings)
	if bar.type == "rune" then
		if bar.bar then
			bar.bar:Hide()
			bar.bar.updatespeed = 1
--			bar.bar:SetScript("OnUpdate", nil)
		end
		bar.total = defaultspec.total
		if not bar.Runes then
			bar.Runes = {}
		end
		for i=1, bar.total do
			local rune = bar.Runes[i]
			if not rune then
				rune = CreateRune(Classbar_BaseName..i, bar, settings)
				bar.Runes[i] = rune
			end
			UpdateTextures(rune, settings, class, i, bar.total) -- might not need bar.total
			rune:ClearAllPoints()
			if i == 1 then
				rune:SetPoint(point, bar, 0, 0)
			else
				rune:SetPoint(point, bar.Runes[i-1], toPoint, spacingX, spacingY)
			end
			rune:Show()
			bar.Deactivate(rune)
		end
		if #bar.Runes > bar.total then
			for i=bar.total+1, #bar.Runes do
				bar.Runes[i]:Hide()
			end
		end
	elseif bar.type == "bar" then
		if bar.Runes then
			for i=1, #bar.Runes do
				bar.Runes[i]:Hide()
			end
		end
		if not bar.bar then
			bar.bar = CreateFrame("Frame", "DUF_PlayerFrame_ClassFrameStatusFrame", bar, "DUF_ClassStatusBarTemplate")
			bar.bar:SetAllPoints()
			bar.bar:EnableMouse(false)
			bar:EnableMouseWheel(false)
			bar.moveparent = true
			bar:RegisterForDrag("LeftButton", "RightButton")
			bar:SetScript("OnDragStart", DUF_Element_OnDragStart)
			bar:SetScript("OnDragStop", DUF_Element_OnDragStop)
		end
		DUF_Initialize_StatusBar(nil, "player", 8, settings)
		local texture = GetTexturePath(settings.textures[1].texture)
--		if texture and texture ~= "" then
			bar.bar.Bar1:SetTexture(texture)
			bar.bar.Bar2:SetTexture(texture)
--		end
		bar.bar:ClearAllPoints()
		bar.bar:SetAllPoints()
		bar.bar.lastvalue = -1
		if _DUF.CLASS_BAR_DEFAULTS[class].onupdate[spec] then
			bar.bar.updatespeed = 1  / _DUF.CLASS_BAR_DEFAULTS[class].onupdate[spec].updatespeed -- set updatespeed first
			bar.bar.updatetimer = 0
			bar.bar.totalelapsed = 0
			bar.bar:SetScript("OnUpdate", _DUF.CLASS_BAR_DEFAULTS[class].onupdate[spec].updatefunc)
			
		end
		bar.bar:Show()
	end
	bar.showlevel = _DUF.CLASS_BAR_DEFAULTS[class].showlevel
	bar:SetScript("OnEvent", DUF_ClassFrame_OnEvent)
	bar:RegisterEvent("PLAYER_TARGET_CHANGED")
	bar:RegisterUnitEvent("UNIT_MAXPOWER", "player")
	bar:RegisterEvent("PLAYER_ENTERING_WORLD")
	local events = _DUF.CLASS_BAR_DEFAULTS[class].extraevents
	if _DUF.CLASS_BAR_DEFAULTS[class].onupdate and _DUF.CLASS_BAR_DEFAULTS[class].onupdate[spec]then
		events = _DUF.CLASS_BAR_DEFAULTS[class].onupdate[spec].extraevents
	end
	if events then
		for _,value in pairs(events) do
			if value.player then
				bar:RegisterUnitEvent(value.event, "player")
			else
				bar:RegisterEvent(value.event)
			end
			if value.showlevel then
				bar.showlevel = value.showlevel
			end
		end
	end
	if bar.showlevel then
		bar.level = UnitLevel("player")
	end
	_DUF:SetupDruid(bar)
	DUF_ClassFrame_UpdateMax(bar)
	DUF_ClassFrame_Update(bar)
	DUF_ClassFrame_OnEvent(bar, "PLAYER_TARGET_CHANGED") -- set up for having a target on init.
	bar:Show()
end

-----------------------------------

DUF_CLASS_CENTERS={ TOP=true, CENTER=true, BOTTOM=true, LEFT=true, RIGHT=true }

--[[
DUF_CLASS_BAR_SPECS={
	ROGUE={ type="rune", total=10, hidealpha=0, power=SPELL_POWER_COMBO_POINTS, [1]=true, [2]=true, [3]=true },
	PALADIN={ type="rune", total=5, hidealpha=0, power=SPELL_POWER_HOLY_POWER, [SPEC_PALADIN_RETRIBUTION]=true },
	MONK={ type="rune", total=6, hidealpha=0, power=SPELL_POWER_CHI, stagger=SPEC_MONK_BREWMASTER, [SPEC_MONK_WINDWALKER]=true, [SPEC_MONK_BREWMASTER] = { type="bar", settings="stagger", hidealpha=1, } },
	MAGE={ type="rune", total=4, hidealpha=0.3, power=SPELL_POWER_ARCANE_CHARGES, [SPEC_MAGE_ARCANE]=true },
	DEATHKNIGHT={ type="rune", total=7, hidealpha=0.3, power=SPELL_POWER_RUNES, [1]=true, [2]=true, [3]=true },
	DRUID={ type="rune", total=5, hidealpha=0, power=SPELL_POWER_COMBO_POINTS, displaypower=SPELL_POWER_ENERGY, [1]=true, [2]=true, [3]=true, [4]=true },
	WARLOCK={ type="rune", total=5, hidealpha=0, power=SPELL_POWER_SOUL_SHARDS, [1]=true, [2]=true, [3]=true },
}
]]--

DUF_CLASS_BAR_SPECS={
	ROGUE={ type="rune", total=10, hidealpha=0, power=Enum.PowerType.ComboPoints, [1]=true, [2]=true, [3]=true },
	PALADIN={ type="rune", total=5, hidealpha=0, power=Enum.PowerType.HolyPower, [SPEC_PALADIN_RETRIBUTION]=true },
	MONK={ type="rune", total=6, hidealpha=0, power=Enum.PowerType.Chi, stagger=SPEC_MONK_BREWMASTER, [SPEC_MONK_WINDWALKER]=true, [SPEC_MONK_BREWMASTER] = { type="bar", settings="stagger", hidealpha=1, } },
	MAGE={ type="rune", total=4, hidealpha=0.3, power=Enum.PowerType.ArcaneCharges, [SPEC_MAGE_ARCANE]=true },
	DEATHKNIGHT={ type="rune", total=7, hidealpha=0.3, power=Enum.PowerType.Runes, [1]=true, [2]=true, [3]=true },
	DRUID={ type="rune", total=5, hidealpha=0, power=Enum.PowerType.ComboPoints, displaypower=Enum.PowerType.Energy, [1]=true, [2]=true, [3]=true, [4]=true },
	WARLOCK={ type="rune", total=5, hidealpha=0, power=Enum.PowerType.SoulShards, [1]=true, [2]=true, [3]=true },
}



_DUF.CLASS_BAR_DEFAULTS = {
	ROGUE = { 
--		individual=false,  -- individual texture for each element
--		individualborder=false,   -- individual border for each element
		shine=true,
		animation="show", -- highlight, cooldown
		extraevents = { [1]={ event="UNIT_POWER_FREQUENT", player=true }, },
		borders={ [1]={
				level="BACKGROUND",
				texture="ComboPoints-PointBg", 
				atlas=true,
--				texture = "Interface\\PLAYERFRAME\\ClassOverlayComboPoints", 
--				texcoords = { left=0.62, right=0.76, top=0.36, bottom=0.62, },  
--No width, heights in borders, just offsets.
--				offsets = { topxos=0, topyos=0, bottomxos=0, bottomyos=0, },
			},
		},
		textures={ [1]={
				level="ARTWORK",
				texture="ComboPoints-ComboPoint", 
				atlas=true,
				blend="BLEND",
--				texture = "Interface\\PLAYERFRAME\\ClassOverlayComboPoints", 
--				texcoords = { left=0.45, right=0.58, top=0.36, bottom=0.62, },
-- No offsets, width, heights in textures				offsets = { topxos=0, topyos=0, bottomxos=0, bottomyos=0, },
			},
		},
	},
	PALADIN = { 
--		individual=true,  -- individual texture for each element
--		individualborder=false,   -- individual border for each element
		shine=true,
		animation="show", -- highlight, cooldown
		extraevents = { [1]={ event="PLAYER_LEVEL_UP", showlevel=PALADINPOWERBAR_SHOW_LEVEL, }, [2]={ event="UNIT_POWER_FREQUENT", player=true }, },
		borders={},
		textures={ [1]={
				level="OVERLAY",
				texture="Interface\\PlayerFrame\\PaladinPowerTextures", 
				texcoords={ left=0.00390625, right=0.14453125, top=0.78906250, bottom=0.96093750, },
				width=36,
				height=22,
			},
			[2]={
				level="OVERLAY",
				texture="Interface\\PlayerFrame\\PaladinPowerTextures", 
				texcoords={ left=0.15234375, right=0.27343750, top=0.78906250, bottom=0.92187500, },
				width=31,
				height=17,
			},
			[3]={
				level="OVERLAY",
				texture="Interface\\PlayerFrame\\PaladinPowerTextures", 
				texcoords={ left=0.28125000, right=0.38671875, top=0.64843750, bottom=0.81250000, },
				width=27,
				height=21,
			},
			[4]={
				level="OVERLAY",
				texture="Interface\\PlayerFrame\\PaladinPowerTextures", 
				texcoords={ left=0.28125000, right=0.38671875, top=0.82812500, bottom=0.92187500, },
				width=27,
				height=12,
			},
			[5]={
				level="OVERLAY",
				texture="Interface\\PlayerFrame\\PaladinPowerTextures", 
				texcoords={ left=0.39453125, right=0.49609375, top=0.64843750, bottom=0.74218750, },
				width=26,
				height=12,
			},
		},
	},
	MONK = { 
--		individual=true,  -- individual texture for each element
--		individualborder=false,   -- individual border for each element
		shine=true,
		animation="show", -- highlight, cooldown
		extraevents = { [1]={ event="UNIT_POWER_FREQUENT", player=true }, },
		onupdate = { [SPEC_MONK_BREWMASTER]={ updatefunc=MonkStaggerUpdate, updatespeed=10 }, },
		borders={},
		borders={ [1]={
				level="BACKGROUND",
				texture="MonkUI-OrbOff", 
				atlas=true,
--				texture = "Interface\\PLAYERFRAME\\ClassOverlayComboPoints", 
--				texcoords = { left=0.62, right=0.76, top=0.36, bottom=0.62, },  
--No width, heights in borders, just offsets.
--				offsets = { topxos=0, topyos=0, bottomxos=0, bottomyos=0, },
			},
		},
		textures={ [1]={
				level="ARTWORK",
				texture="MonkUI-LightOrb", 
				atlas=true,
				blend="BLEND",
--				texture = "Interface\\PLAYERFRAME\\ClassOverlayComboPoints", 
--				texcoords = { left=0.45, right=0.58, top=0.36, bottom=0.62, },
-- No offsets, width, heights in textures				offsets = { topxos=0, topyos=0, bottomxos=0, bottomyos=0, },
			},
		},
	},
---- MAGE
	MAGE = { 
		shine=true,
		animation="highlight", -- highlight, cooldown
		extraevents={ [1]={ event="UNIT_POWER_FREQUENT", player=true }, },
		borders={},
		textures={ [1]={
				level="ARTWORK",
				texture="Mage-ArcaneCharge", 
				atlas=true,
				useatlassize=true,
				blend="ADD",
			},
		},
	},
---- DEATHKNIGHT
	DEATHKNIGHT = { 
		shine=true,
		animation="cooldown", -- highlight, cooldown
		extraevents={ [1]={ event="RUNE_POWER_UPDATE" }, },
--, [2]={ event="RUNE_TYPE_UPDATE" } -- maybe later, it's just a flash ATM		
		borders={ [1]={
				level="BACKGROUND",
				texture="Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Ring",
				color={ r=0.6, g=0.6, b=0.6 },

--				level = "BACKGROUND",
--				texture = "ComboPoints-PointBg", 
--				atlas=true,
			},
		},
		textures={ [1]={
				level="ARTWORK",
				texture="Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-SingleRune", 
			},
		},
	},
---- DRUID
	DRUID = { 
		shine=true,
		animation="show", -- highlight, cooldown
		extraevents = { [1]={ event="UNIT_DISPLAYPOWER", player=true }, },
		borders={ [1]={
				level="BACKGROUND",
				texture="ComboPoints-PointBg", 
				atlas=true,
			},
		},
		textures={ [1]={
				level="ARTWORK",
				texture="ComboPoints-ComboPoint", 
				atlas=true,
				blend="BLEND",
			},
		},
	},
---- WARLOCK
	WARLOCK = { 
		shine=true,
		animation="highlight",
		extraevents = { [1]={ event="UNIT_POWER_FREQUENT", player=true }, },
		borders={ [1]={
				level="BACKGROUND",
				texture="Interface\\PlayerFrame\\UI-WarlockShard", 
				texcoords={ left=0.01562500, right=0.82812500, top=0.60937500, bottom=0.83593750, },
				offsets={ topxos=-10, topyos=7, bottomxos=29, bottomyos=-10, },
			},
		},
		textures={ [1]={
				level="ARTWORK",
				texture="Interface\\PlayerFrame\\UI-WarlockShard",
				texcoords={ left=0.01562500, right=0.28125000, top=0.00781250, bottom=0.13281250, },
			},
		},
	},
}
