local AddOn, _DUF = ...
local function DUF_GetPvpIcon(factionGroup)
	if (factionGroup == "Neutral") then
		return "FFA";
	else
		return factionGroup;
	end
end

------------------------------------------------------- GLOBAL FUNCTIONS -----------------------------------------

function _DUF:DUF_LeaderIcon_Update(unit)
	if (not DUF_INITIALIZED) then return; end
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].LeaderIcon.hide) then return; end
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_LeaderIcon"].Icon
	local inParty = GetNumGroupMembers() + GetNumSubgroupMembers();
	if (DUF_OPTIONS_VISIBLE) then
		icon:Show();
	elseif (UnitIsGroupLeader(unit) and inParty > 0) then
		icon:Show();
	else
		icon:Hide();
	end
end

function _DUF:DUF_LootIcon_Update(unit)
	if (not DUF_INITIALIZED) then return; end
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].LootIcon.hide) then return; end
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_LootIcon"].Icon
	if (DUF_OPTIONS_VISIBLE) then
		icon:Show();
		return;
	end
	icon:Hide();
	if (unit == "target") then return; end

	local looterNum;
	if (unit == "player") then
		looterNum = 0;
	elseif (unit == "party1") then
		looterNum = 1;
	elseif (unit == "party2") then
		looterNum = 2;
	elseif (unit == "party3") then
		looterNum = 3;
	elseif (unit == "party4") then
		looterNum = 4;
	end

	local __, lootMaster = GetLootMethod();
	local inParty = GetNumGroupMembers() + GetNumSubgroupMembers();
	if ( lootMaster == looterNum and (inParty > 0) ) then
		icon:Show();
	end
end

function _DUF:DUF_PVPIcon_Update(unit)
	if (not DUF_INITIALIZED) then return; end
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].PVPIcon.hide) then return; end
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_PVPIcon"].Icon
	icon:Show();
	local factionGroup = UnitFactionGroup(unit);
	if ( UnitIsPVPFreeForAll(unit) ) then
		icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA");
	elseif ( factionGroup and UnitIsPVP(unit)) then
		icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..DUF_GetPvpIcon(factionGroup));
	elseif (DUF_OPTIONS_VISIBLE) then
			factionGroup = UnitFactionGroup("player");
			icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..DUF_GetPvpIcon(factionGroup));
			return;
	else
		icon:Hide();
	end
end

function _DUF:DUF_RoleIcon_Update(unit)
	if (not DUF_INITIALIZED or InCombatLockdown()) then return; end -- Fizzlemizz: Comment: InCombatLockdown()  = 5.0.4 change
	local texture = _G[DUF_FRAME_DATA[unit].frame.."_RoleIcon"].Icon
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_RoleIcon"];
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].RoleIcon.hide) then 
		texture:Hide()
		icon:Hide(); 
		return;
	end
	texture:SetTexture("Interface\\AddOns\\DiscordUnitFrames\\CustomTextures\\tank"); -- Default to tank
	local role = UnitGroupRolesAssigned(unit);
	if (role == "HEALER") then
		texture:SetTexture("Interface\\AddOns\\DiscordUnitFrames\\CustomTextures\\healer");
	elseif (role == "DAMAGER") then
		texture:SetTexture("Interface\\AddOns\\DiscordUnitFrames\\CustomTextures\\damage");
	end
	if (role == "NONE" and not DUF_OPTIONS_VISIBLE) then
		texture:Hide();
		icon:Hide();
	else
		texture:Show();
		icon:Show();
	end
end

function _DUF:DUF_PhaseIcon_Update(unit)
	if not DUF_INITIALIZED or not unit then return end
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].PhaseIcon
	if settings.hide then return end
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_PhaseIcon"]
	if (not UnitInPhase(unit) and UnitIsConnected(unit)) or DUF_OPTIONS_VISIBLE then
		frame:SetAlpha(1)
	else
		frame:SetAlpha(0)
	end
end

function _DUF:DUF_StatusIcon_Update(unit)
	if (not DUF_INITIALIZED) then return; end
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusIcon.hide) then return; end
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_StatusIcon"].Icon
	icon:Show();
	if (DUF_OPTIONS_VISIBLE) then
		icon:SetTexCoord(.5625, .9, .08, .4375);
		icon:SetTexture("Interface/CharacterFrame/UI-StateIcon");
	elseif (not UnitIsConnected(unit)) then
		icon:SetTexture("Interface/CharacterFrame/Disconnect-Icon");
		icon:SetTexCoord(0, 1, 0, 1);
	elseif (UnitHealth(unit) <= 0 or UnitIsGhost(unit)) then
		icon:SetTexture("Interface/TargetingFrame/UI-TargetingFrame-Skull");
		icon:SetTexCoord(0, 1, 0, 1);
	elseif (unit == "player" and DL_ATTACKING) then
		icon:SetTexCoord(.5625, .9, .08, .4375);
		icon:SetTexture("Interface/CharacterFrame/UI-StateIcon");
	elseif (unit == "player" and DL_INCOMBAT) then
		icon:SetTexCoord(.5625, .9, .08, .4375);
		icon:SetTexture("Interface/CharacterFrame/UI-StateIcon");
	elseif (UnitAffectingCombat(unit)) then
		icon:SetTexCoord(.5625, .9, .08, .4375);
		icon:SetTexture("Interface/CharacterFrame/UI-StateIcon");
	elseif (unit == "player" and IsResting()) then
		icon:SetTexCoord(.0625, .4375, .0625, .4375);
		icon:SetTexture("Interface/CharacterFrame/UI-StateIcon");
	else
		icon:Hide();
	end
end
