local AddOn, _DUF = ...
-- Legion local AnticipationId = 114015
-- Legion local AnticipationName = "Anticipation"
local strfind = string.find

local function DUF_Slash_Handler(msg)
	local command, param;
	local index = string.find(msg, " ");
	if( index) then
		command = string.sub(msg, 1, (index - 1));
		param = string.sub(msg, (index + 1)  );
	else
		command = msg;
	end

	if InCombatLockdown() then
		message("You cannot configure Discord Unit Frames while in combat.") 
	else
		if (not DUF_Options) then
			DUF_Load_Options(0); 
		end
		if (DUF_Options:IsVisible()) then
			DUF_Options:Hide();
		else
			DUF_Options:Show();
		end
	end
end

local function DUF_Initialize_PartyFrames()
	for unit, value in pairs(DUF_PARTY_DATA) do
		local unitFrame = _G[DUF_FRAME_DATA[unit].frame];
		local children = { unitFrame:GetChildren() }
		for _, child in ipairs(children) do
			child.menu = unitFrame.menu
		end
		DUF_Initialize_Frame(unitFrame, unit);
		DUF_Initialize_Portrait(unit);
		DUF_Initialize_ClassIcon(unit);
		DUF_Initialize_RaceIcon(unit);
		DUF_Initialize_RoleIcon(unit);
		for i=1, _DUF.DUF_NUM_OTHERS_TEXTBOXES do
			DUF_Initialize_TextBox(0, unit, i); 
		end
		for i=1,_DUF.DUF_NUM_STATUSBARS do
			DUF_Initialize_StatusBar(0, unit, i, nil, debug); 
		end
		DUF_Initialize_PVPIcon(unit);
		DUF_Initialize_StatusIcon(unit);
		DUF_Initialize_LootIcon(unit);
		DUF_Initialize_LeaderIcon(unit);
		DUF_Initialize_Buffs(unitFrame, unit);
		DUF_Initialize_Debuffs(unitFrame, unit);
	end
end

---------------------------------*** End of locals ***-------------------------------------------------

function DUF_Load_Options()
	if (DUF_Options) then return; end
	DUF_GENERALPROPERTIES = nil;
	UIParentLoadAddOn("DiscordUnitFramesOptions");
	DL_Set_OptionsTitle("DUF_Options", "DiscordUnitFrames\\Icons\\title.tga", _DUF.DUF_VERSION .. _DUF.DUF_SUBVERSION);
	_DUF:DUF_Initialize_VariablesFrame();
	DUF_FRAME_INDEX = "player";
	DUF_SelectFrame_OnClick(DUF_Options_Self, "player", DUF_Options_Self);
	DUF_Options_SelectFrameOptions();
	DUF_OPTIONS_SELECTED = "DUF_ElementOptions_BaseSelect";
	DUF_ElementOptions_BaseSelect:SetNormalFontObject(GameFontNormal);
	DUF_ElementOptions_BaseSelect:SetBackdropColor(1, 1, 0);
	DUF_ElementOptions_BaseSelect:SetBackdropBorderColor(1, 1, 1);
	DUF_Update_SavedSettings();
	DUF_Options_SetScale();
	DUF_Init_ContextColors();
	DUF_Init_MiscOptions();
end

function DUF_Main_OnEvent(self, event, ...)
	if (not DUF_INITIALIZED) then return; end
	local arg1, arg2 = ...
	if (event == "GROUP_ROSTER_UPDATE" or event == "RAID_ROSTER_UPDATE" or event == "PLAYER_ENTERING_WORLD") then
		DUF_Main_UpdatePartyMembers();
	elseif event == "PLAYER_FLAGS_CHANGED" then
		if DUF_Settings[DUF_INDEX].hidepartyinraid and _DUF.isinraid then return end
		if not strfind(arg1, "raid") then
			DUF_Main_UpdatePartyMembers();
		end
	elseif event == "UNIT_PHASE" then
		--for unit, value in pairs(DUF_FRAME_DATA) do
	 	for i=1,4 do
			_DUF:DUF_PhaseIcon_Update("party"..i)
		end
	elseif (event == "UNIT_TARGET") then
		if arg1 == "target" then
			DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
		end
	elseif (event == "PLAYER_TARGET_CHANGED") then
		if (DL_UnitName("target")) then
			if (not DUF_Settings[DUF_INDEX].target.hide) then
					DUF_UnitFrame_OnShow(DUF_TargetFrame)
					DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
					_DUF:DUF_Set_EliteTexture();
			end
			if (DL_UnitName("targettarget")) then
				if (not DUF_Settings[DUF_INDEX].targettarget.hide) then
					if (DUF_Settings[DUF_INDEX].hidetargettarget) then
						if (GetNumGroupMembers() + GetNumSubgroupMembers() > 0) then
								DUF_SetFrameUnitWatch(DUF_TargetOfTargetFrame,true);
								DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
						else
								DUF_SetFrameUnitWatch(DUF_TargetOfTargetFrame,false);
						end
					else
						DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
					end
				end
			end
		end
	elseif (event == "PLAYER_XP_UPDATE" or event == "PLAYER_UPDATE_RESTING") then
		if (not DUF_Settings[DUF_INDEX].player.StatusBar[3].trackRep) then
			_DUF:DUF_XPBar_Update();
		end
	elseif (event == "UPDATE_FACTION") then
		if (DUF_Settings[DUF_INDEX].player.StatusBar[3].trackRep) then
			_DUF:DUF_XPBar_Update();
		end
	elseif (event == "PLAYER_LEVEL_UP") then
		_DUF:DUF_XPBar_Update();
	elseif (event == "UNIT_PET_EXPERIENCE") then
		_DUF:DUF_PetXPBar_Update();
	elseif (event == "PLAYER_REGEN_DISABLED") then
		if (DUF_OPTIONS_VISIBLE) then
      			DUF_Options:Hide()
    		end
    	elseif (event == "UPDATE_SHAPESHIFT_FORM") then  
    		if not _DUF.DUF_ISDRUID then return; end
    			_DUF:DUF_DruidManaBar_Update_Display()
	elseif event == "UNIT_DISPLAYPOWER" then
		_DUF:DUF_DruidManaBar_Update_Display()
	elseif (event == "PLAYER_UNGHOST") then
		for unit, value in pairs(DUF_FRAME_DATA) do
			if (unit and not UnitIsDeadOrGhost(unit))then
				_DUF:DUF_Set_Portrait(unit)
			end
		end
	end
end

function DUF_Main_OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_COMBO_POINTS") -- For textboxes.See variables.lua
	self:RegisterEvent("PLAYER_FLAGS_CHANGED");
	self:RegisterEvent("PLAYER_LEVEL_UP");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("UNIT_TARGET");
	self:RegisterEvent("PLAYER_UPDATE_RESTING");
	self:RegisterEvent("PLAYER_XP_UPDATE");
	self:RegisterEvent("UNIT_PHASE");
	self:RegisterEvent("UNIT_PET_EXPERIENCE");
	self:RegisterEvent("UPDATE_EXHAUSTION");
	self:RegisterEvent("UPDATE_FACTION");
	self:RegisterEvent("ARTIFACT_XP_UPDATE")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("PLAYER_UNGHOST");
	self:RegisterEvent("UNIT_NAME_UPDATE");
--	self:RegisterUnitEvent("UNIT_POWER_FREQUENT", "player", "vehicle");
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	DiscordLib_RegisterInitFunc(_DUF.DUF_CoreInitialize);
	SLASH_DUF1 = "/duf";
	SLASH_DUF2 = "/discordunitframes";
	SlashCmdList["DUF"] = function(msg)
		DUF_Slash_Handler(msg)
	end
end

function DUF_Main_OnUpdate(self, elapsed)
	if (not DUF_INITIALIZED) then return; end
	if (not self.timer) then 
		self.timer = DUF_Settings[DUF_INDEX].updatespeed; 
		self.updatespeed = self.timer
	end
	self.timer = self.timer - elapsed;
	if (self.timer < 0) then
		self.timer = self.updatespeed
		local min,max = GetPetExperience();
		if (min ~= self.minpetxp or max ~= self.maxpetxp) then
			self.minpetxp = min;
			self.maxpetxp = max;
			_DUF:DUF_PetXPBar_Update();
		end
	end
end

function DUF_Main_UpdatePartyMembers()
	local force_hide_party = DUF_Settings[DUF_INDEX].party.hide or (DUF_Settings[DUF_INDEX].hidepartyinraid and IsInRaid())
	local force_hide_partypet = force_hide_party or DUF_Settings[DUF_INDEX].partypet.hide
	local frame
	for i=1,4 do
		frame = _G["DUF_PartyFrame"..i]
		DUF_SetFrameUnitWatch(frame, not force_hide_party)
		frame = _G["DUF_PartyPetFrame"..i]
--		DUF_SetFrameUnitWatch(frame, true)
		DUF_SetFrameUnitWatch(frame, not force_hide_partypet)
	end
 	if (InCombatLockdown()) then
 		return
 	end
--	DUF_Initialize_PartyFrames() -- DUF_Initialize_AllFrames()
 	if (not DUF_Settings[DUF_INDEX].party.hide) then
 		if (DUF_Settings[DUF_INDEX].hidepartyinraid) then
 			if (IsInRaid()) then
 				_DUF.isinraid = true
			 	for i=1,4 do
			 		frame = _G["DUF_PartyFrame"..i]
			 		frame:Hide()
			 		frame = _G["DUF_PartyPetFrame"..i]
			 		frame:Hide()
--			 		_G["DUF_PartyFrame"..i]:Hide();
--			 		_G["DUF_PartyPetFrame"..i]:Hide();
			 	end
 			else
 				_DUF.isinraid = nil
 				DUF_Initialize_PartyFrames() -- DUF_Initialize_AllFrames()
		 		for i=1, GetNumSubgroupMembers() do
		 			_G["DUF_PartyFrame"..i]:Show();
		 			if (DL_UnitName("partypet"..i) and UnitExists("partypet"..i) and (not DUF_Settings[DUF_INDEX].partypet.hide)) then
		 				_G["DUF_PartyPetFrame"..i]:Show();
		 			end
		 		end
 			end
		else
			DUF_Initialize_PartyFrames() -- DUF_Initialize_AllFrames() -- Need to check is DUF_Initialize_PartyFrames needs to be part of DUF_Main_UpdatePartyMembers
 		end
	end
end

function DUF_Toggle_ElementsLock()
	if (DUF_ELEMENTS_UNLOCKED) then
		DUF_ELEMENTS_UNLOCKED = nil;
		if (DUF_Options) then
			DUF_Options_ToggleElements:SetText(DUF_TEXT.UnlockElements);
		end
		if (not DUF_FRAMES_UNLOCKED) then
			if (GB_PlayerBar) then
				DUF_HIDING_CLICKBOXES = nil;
				if (not GB_Settings[GB_INDEX].hideClickboxes) then
					GB_PlayerClickbox:Show();
					if (DL_UnitName("pet")) then
						GB_Pet0Clickbox:Show();
					end
					for i=1, GetNumSubgroupMembers() do
						_G["GB_Party"..i.."Clickbox"]:Show();
						if (DL_UnitName("partypet"..i)) then
							_G["GB_Pet"..i.."Clickbox"]:Show();
						end
					end
					if (DL_UnitName("target")) then
						GB_TargetClickbox:Show();
					end
				end
			end
		end
	else
		DUF_ELEMENTS_UNLOCKED = 1;
		if (DUF_Options) then
			DUF_Options_ToggleElements:SetText(DUF_TEXT.LockElements);
		end
		if (GB_PlayerBar) then
			DUF_HIDING_CLICKBOXES = true;
			for _, frame in pairs(GB_CLICKBOXES) do
				_G[frame]:Hide();
			end
		end
	end
end

function DUF_Toggle_FramesLock()
	if (DUF_FRAMES_UNLOCKED) then
		DUF_FRAMES_UNLOCKED = nil;
		if (DUF_Options) then
			DUF_Options_ToggleFrames:SetText(DUF_TEXT.UnlockFrames);
		end
		if (not DUF_ELEMENTS_UNLOCKED) then
			if (GB_PlayerBar) then
				DUF_HIDING_CLICKBOXES = nil;
				if (not GB_Settings[GB_INDEX].hideClickboxes) then
					GB_PlayerClickbox:Show();
					if (DL_UnitName("pet")) then
						GB_Pet0Clickbox:Show();
					end
					for i=1, GetNumSubgroupMembers() do
						_G["GB_Party"..i.."Clickbox"]:Show();
						if (DL_UnitName("partypet"..i)) then
							_G["GB_Pet"..i.."Clickbox"]:Show();
						end
					end
					if (DL_UnitName("target")) then
						GB_TargetClickbox:Show();
					end
				end
			end
		end
	else
		DUF_FRAMES_UNLOCKED = 1;
		if (DUF_Options) then
			DUF_Options_ToggleFrames:SetText(DUF_TEXT.LockFrames);
		end
		if (GB_PlayerBar) then
			DUF_HIDING_CLICKBOXES = true;
			for _, frame in pairs(GB_CLICKBOXES) do
				_G[frame]:Hide();
			end
		end
	end
end
